#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "alloc-util.h"
#include "dropin.h"
#include "generator.h"
#include "logind.h"
#include "path-util.h"

static int read_manager_configuration(char **user_tasks_max) {
        Manager m = {};
        int r;

        manager_reset_config(&m);
        m.user_tasks_max = NULL;

        r = manager_parse_config_file(&m);
        if (r < 0)
                return log_warning_errno(r, "Failed to parse logind.conf: %m");

        if (!m.user_tasks_max)
                return 0;

        *user_tasks_max = m.user_tasks_max;
        return 1;
}

static int run(const char *dest, const char *dest_early, const char *dest_late) {
        _cleanup_free_ char *p = NULL;
        char *user_tasks_max;
        int r = 0;

        /* FIXME: DEFINE_MAIN_GENERATOR_FUNCTION() is supposed to open the log
         * for us but it doesn't work. Use kmsg to make sure that our warning
         * ends up in the journal. */
        log_set_target(LOG_TARGET_KMSG);
        log_parse_environment();
        log_open();

        umask(0022);

        r = read_manager_configuration(&user_tasks_max);
        if (r == 0)
                return EXIT_SUCCESS;
        if (r < 0)
                return EXIT_FAILURE;

        p = path_join(dest, "user-.slice.d", "50-limits.conf");
        if (!p)
                return EXIT_FAILURE;

        log_warning("Creating %s to keep compatibility\n"
                    "Please copy the snippet in /etc/systemd/system/user-.slice.d/ and remove any uses of UserTasksMax=\n", p);

        r = write_drop_in_format(dest, "user-.slice", 50, "limits",
                                 "# Automatically generated by logind-compat-tasks-max-generator\n\n"
                                 "[Slice]\nTasksMax=%s", user_tasks_max);

        return r < 0 ? EXIT_FAILURE : EXIT_SUCCESS;
}

DEFINE_MAIN_GENERATOR_FUNCTION(run);
